/*
	Copyright (C) 2005 by Ruben Henner Zilibowitz <rzilibowitz@users.sourceforge.net>
	Part of the Toy Cars Project http://toycars.sourceforge.net

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License.
	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY.

	See the COPYING file for more details.
*/

/*
 *  Created by Ruben on Wed Apr 21 2004.
 */

#ifndef CONTACT_H
#define CONTACT_H

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include "Tuple.h"

// forwards declaration
class RigidBody;

// Contact class
// When a collision between two geometries is detected, the information about the collision may
// be stored in a Contact class which can then calculate the proper collision response and counter
// interpenetration between bodies.

class Contact
{
public:
	Contact(RigidBody* inBodyA, RigidBody* inBodyB, const Tuple& inPa, const Tuple& inPb, const Tuple& inNormal);
    
    bool resolve_collision();
    void resolve_overlap();
    double compute_velocity();
	
	Tuple getLoc() { return (pa+pb)*0.5; }
	
protected:
    RigidBody *bodyA, *bodyB;
    const Tuple pa, pb, normal, ra, rb;
    const double K;
};

const double kCollisionRestitution = 0.5;
const double kCollisionThreshold = -0.1;
const double kCollisionSeperation = 0.15;

#endif
